"""
Tests the redirects plugin
"""
from pathlib import Path

import pytest
from conftest import set_directory

from markata import Markata
from markata.plugins import redirects


@pytest.mark.parametrize(
    "tmp_files, old, new",
    [
        ({"static/_redirects": "old new"}, "old", "new"),
        ({"static/_redirects": "post_one post-one"}, "post_one", "post-one"),
    ],
    indirect=["tmp_files"],
)
def test_redirect_exists(tmp_files: Path, old: str, new: str) -> None:
    "ensure that the default workflow works"
    with set_directory(tmp_files):
        m = Markata()
        redirects.save(m)
        redirect_file = Path("markout") / old / "index.html"
        assert redirect_file.exists()
        assert (
            f'<meta http-equiv="Refresh" content="0; url=\'{new}\'" />'
            in redirect_file.read_text()
        )


@pytest.mark.parametrize(
    "tmp_files, old",
    [
        ({"static/_redirects": "posts/* pages/:splat/"}, "posts"),
        ({"static/_redirects": "posts/*/2000 pages/:splat/2000"}, "posts"),
    ],
    indirect=["tmp_files"],
)
def test_redirect_ignore_splat(tmp_files: Path, old: str) -> None:
    "splats cannot be supported statically, test that they are ignored"
    with set_directory(tmp_files):
        m = Markata()
        redirects.save(m)
        redirect_file = Path("markout") / old / "index.html"
        assert not redirect_file.exists()


@pytest.mark.parametrize(
    "tmp_files, old",
    [
        ({"static/_redirects": "/blog/ /blog/404.html 404"}, "posts"),
        ({"static/_redirects": "/blog/ /blog/301.html 301"}, "posts"),
        ({"static/_redirects": "/blog/ /blog/200.html 200"}, "posts"),
    ],
    indirect=["tmp_files"],
)
def test_redirect_ignore_more_params(tmp_files: Path, old: str) -> None:
    "status codes cannot be supported statically as they are issued by the server"
    with set_directory(tmp_files):
        m = Markata()
        redirects.save(m)
        redirect_file = Path("markout") / old / "index.html"
        assert not redirect_file.exists()


@pytest.mark.parametrize(
    "tmp_files, redirect_file, old, new",
    [
        ({"static/_redirects": "old new"}, "static/_redirects", "old", "new"),
        ({"_redirects": "old new"}, "_redirects", "old", "new"),
        ({"assets/_redirects": "old new"}, "assets/_redirects", "old", "new"),
    ],
    indirect=["tmp_files"],
)
def test_redirect_configure_redirect_file(
    tmp_files: Path, redirect_file: str, old: str, new: str
) -> None:
    "ensure that the redirects file can be configured"
    with set_directory(tmp_files):
        m = Markata()
        m.config["redirects"] = redirect_file
        redirects.save(m)
        redirect_html = Path("markout") / old / "index.html"
        assert redirect_html.exists()
        assert (
            f'<meta http-equiv="Refresh" content="0; url=\'{new}\'" />'
            in redirect_html.read_text()
        )


@pytest.mark.parametrize(
    "tmp_files, redirect_template, old, new",
    [
        (
            {
                "static/_redirects": "old new",
                "templates/redirect_template": "{{ original }} is now {{ new }}",
            },
            "templates/redirect_template",
            "old",
            "new",
        ),
        (
            {
                "static/_redirects": "old new",
                "plugins/templates/redirect_template.html": "{{ original }} is now {{ new }}",
            },
            "plugins/templates/redirect_template.html",
            "old",
            "new",
        ),
    ],
    indirect=["tmp_files"],
)
def test_redirect_custom_template(
    tmp_files: Path, redirect_template: str, old: str, new: str
) -> None:
    "ensure the template can be configured"
    with set_directory(tmp_files):
        m = Markata()
        m.config["redirect_template"] = redirect_template
        redirects.save(m)
        redirect_file = Path("markout") / old / "index.html"
        assert redirect_file.exists()
        assert f"{old} is now {new}" in redirect_file.read_text()


@pytest.mark.parametrize(
    "tmp_files, old, new",
    [
        (
            {
                "static/_redirects": "",
            },
            "old",
            "new",
        ),
    ],
    indirect=["tmp_files"],
)
def test_redirect_empty(tmp_files: Path, old: str, new: str) -> None:
    "ensure empty redirects files work"
    with set_directory(tmp_files):
        m = Markata()
        redirects.save(m)


def test_redirect_file_missing(tmpdir: Path) -> None:
    "ensure missing redirects file works"
    with set_directory(tmpdir):
        m = Markata()
        redirects.save(m)



"""
Creates redirects for times when your backend server can't.

## Configuration

Enable the redirect hook by adding it to your list of hooks.

``` toml
[markata]

# Were you keep static assets to copy into the project, default is static
# the assets_dir will set the default _redirects file directory
assets_dir = "static"

# You can override the default redirects file location
redirects = static/_redirects

hooks = [
   "markata.plugins.redirects", # creates redirects from static/_redirects file
   "markata.plugins.copy_assets", # copies your static assets into the output_dir (default: `markout`)
  ...
]
```

## Syntax

Your `_redirects` file is a simplified version of what services like cloudflare
pages or netlify use.  In fact you can use the same redirects file!

Here is an example that will redirect `/old` to `/new` and `/CHANGELOG` to
`/changelog`

```
/old        /new
/CHANGELOG  /changelog
```

## Limitations
_no splats_

Since it is static generated this plugin cannot cover *'s.  * or splat
redirects need to be taken care of server side.  It also cannot change the http
code, this is only

## Features

The features of markata.plugins.redirect is pretty limited since it is
implemented only as a static page.  Other features require server side
implementation.

| Feature                             | Support | Example                                                         | Notes                                                                                             |
| ----------------------------------- | ------- | --------------------------------------------------------------- | ------------------------------------------------------------------------------------------------- |
| Force                               | Yes     | `/pagethatexists /otherpage`                                    | Creates an index.html with http-equiv and canonical                                               |
| Redirects (301, 302, 303, 307, 308) | No      | `/home / 301`                                                   | Ignored, requires server side implementation                                                      |
| Rewrites (other status codes)       | No      | `/blog/* /blog/404.html 404`                                    | ...                                                                                               |
| Splats                              | No      | `/blog/* /blog/:splat`                                          | ...                                                                                               |
| Placeholders                        | No      | `/blog/:year/:month/:date/:slug /news/:year/:month/:date/:slug` | ...                                                                                               |
| Query Parameters                    | No      | `/shop id=:id /blog/:id 301`                                    | ...                                                                                               |
| Proxying                            | No      | `/blog/* https://blog.my.domain/:splat 200`                     | ...                                                                                               |
| Domain-level redirects              | No      | `workers.example.com/* workers.example.com/blog/:splat 301`     | ...                                                                                               |
| Redirect by country or language     | No      | `/ /us 302 Country=us`                                          | ...                                                                                               |
| Redirect by cookie                  | No      | `/* /preview/:splat 302 Cookie=preview`                        | ...                                                                                               |

> Compare with [cloudflare-pages](https://developers.cloudflare.com/pages/platform/redirects/)

!!! tip
    If you have a public site, pair this up with
    [ahrefs](https://app.ahrefs.com/dashboard) to keep up with pages that have
    moved without you realizing.

"""
from dataclasses import asdict, dataclass
from pathlib import Path
from typing import TYPE_CHECKING

from jinja2 import Template

from markata.hookspec import hook_impl

if TYPE_CHECKING:
    from markata import Markata

DEFAULT_REDIRECT_TEMPLATE = Path(__file__).parent / "default_redirect_template.html"


@dataclass
class Redirect:
    "DataClass to store the original and new url"
    original: str
    new: str


@hook_impl
def save(markata: "Markata") -> None:
    """
    saves an index.html in the directory called out by the redirect.
    """
    assets_dir: str = str(markata.config.get("assets_dir", "static"))
    redirects_file = Path(
        str(markata.config.get("redirects", Path(assets_dir) / "_redirects"))
    )
    if redirects_file.exists():
        raw_redirects = redirects_file.read_text().split("\n")
    else:
        raw_redirects = []

    redirects = [
        Redirect(*s)
        for r in raw_redirects
        if "*" not in r and len(s := r.split()) == 2 and not r.strip().startswith("#")
    ]

    if "redirect_template" in markata.config:
        template_file = Path(str(markata.config.get("redirect_template")))
    else:
        template_file = DEFAULT_REDIRECT_TEMPLATE
    template = Template(template_file.read_text())

    output_dir = Path(markata.config["output_dir"])  # type: ignore
    output_dir.mkdir(parents=True, exist_ok=True)

    for redirect in redirects:
        file = output_dir / redirect.original.strip("/") / "index.html"
        file.parent.mkdir(parents=True, exist_ok=True)
        file.write_text(template.render(**asdict(redirect), config=markata.config))



import pytest

from markata import Markata
from markata.plugins import flat_slug


@pytest.mark.parametrize(
    "article, slug",
    [
        ({"slug": "post_one"}, "post-one"),
        ({"slug": "post-one"}, "post-one"),
        ({"slug": "post one"}, "post-one"),
        ({"slug": "POST_ONE"}, "post-one"),
        ({"slug": "posts/post_one"}, "posts/post-one"),
        ({"slug": "posts/post-one"}, "posts/post-one"),
        ({"slug": "posts/post one"}, "posts/post-one"),
        ({"slug": "posts/POST_ONE"}, "posts/post-one"),
        ({"slug": "POSTS/post_one"}, "posts/post-one"),
        ({"slug": "/posts/post_one"}, "/posts/post-one"),
    ],
)
def test_flat_slug(article, slug) -> None:
    "ensure that the explicit workflow works"
    m = Markata()
    m.articles = [article]
    flat_slug.pre_render(m)

    assert m.articles[0]["slug"] == slug


@pytest.mark.parametrize(
    "article, slug",
    [
        ({"path": "post_one.md"}, "post-one"),
        ({"path": "post-one.md"}, "post-one"),
        ({"path": "post one.md"}, "post-one"),
        ({"path": "POST_ONE.md"}, "post-one"),
        ({"path": "posts/post_one.md"}, "posts/post-one"),
        ({"path": "posts/post-one.md"}, "posts/post-one"),
        ({"path": "posts/post one.md"}, "posts/post-one"),
        ({"path": "posts/POST_ONE.md"}, "posts/post-one"),
        ({"path": "POSTS/post_one.md"}, "posts/post-one"),
        ({"path": "/posts/post_one.md"}, "/posts/post-one"),
    ],
)
def test_flat_slug_path(article, slug) -> None:
    "ensure that the second most explicit workflow works"
    m = Markata()
    m.articles = [article]
    flat_slug.pre_render(m)


@pytest.mark.parametrize(
    "article, slug",
    [
        ({"title": "post_one"}, "post-one"),
        ({"title": "post-one"}, "post-one"),
        ({"title": "post one"}, "post-one"),
        ({"title": "POST_ONE"}, "post-one"),
        ({"title": "/post_one"}, "post-one"),
        ({"title": "posts/post_one"}, "posts/post-one"),
        ({"title": "posts/post-one"}, "posts/post-one"),
        ({"title": "posts/post one"}, "posts/post-one"),
        ({"title": "posts/POST_ONE"}, "posts/post-one"),
        ({"title": "POSTS/post_one"}, "posts/post-one"),
        ({"title": "/posts/post_one"}, "/posts/post-one"),
    ],
)
def test_flat_slug_title(article, slug) -> None:
    "ensure that the second most explicit workflow works"
    m = Markata()
    m.articles = [article]
    flat_slug.pre_render(m)



from pathlib import Path
from typing import TYPE_CHECKING

from markata.hookspec import hook_impl

if TYPE_CHECKING:
    from markata import Markata


@hook_impl(tryfirst=True)
def pre_render(markata: "Markata") -> None:
    """
    Sets the article slug if one is not already set in the frontmatter.
    """
    subroute = markata.config.get("subroute", "")
    for article in markata.iter_articles(description="creating slugs"):
        slug = Path(article.get("slug", Path(article["path"]).stem))
        article["slug"] = slug.parent / Path(subroute) / slug.stem



"""
Setup Logging hook sets up the RichHandler for pretty console logs, and file logs to the configured markata's configured `log_dir`, or
`output_dir/_logs` if `log_dir` is not configured.  The log file will be named after the `<levelname>.log`

## The log files

There will be 6 log files created based on log level and file type.

```
markout/_logs
├── debug
│   └── index.html
├── debug.log
├── info
│   └── index.html
├── info.log
├── warning
│   └── index.html
└── warning.log
```

## Configuration

Ensure that setup_logging is in your hooks.  You can check if `setup_logging`
is in your hooks by running `markata list --hooks` from your terminal and
checking the output, or creating an instance of `Markata()` and checking the
`Markata().hooks` attribute.  If its missing or you wan to be more explicit,
you can add `setup_logging` to your `markata.toml` `[markata.hooks]`.

``` toml
[markata]

# make sure its in your list of hooks
hooks=[
   "markata.plugins.setup_logging",
   ]
```

## Log Template
``` toml
[markata]

# make sure its in your list of hooks
hooks=[
   "markata.plugins.setup_logging",
   ]

# point log template to the path of your logging template
log_template='templates/log_template.html'
```

You can see the latest default `log_template` on
[GitHub](https://github.com/WaylonWalker/markata/blob/main/markata/plugins/default_log_template.html)

## Disable Logging

If you do not want logging, you can explicityly disable it by adding it to your
`[markata.disabled_hooks]` array in your `[markata.toml]`

``` toml
[markata]

# make sure its in your list of hooks
disabled_hooks=[
   "markata.plugins.setup_logging",
   ]
```

"""
import datetime
import logging
import sys
from pathlib import Path
from typing import TYPE_CHECKING

from jinja2 import Template, Undefined
from rich.logging import RichHandler

from markata.hookspec import hook_impl

if TYPE_CHECKING:
    from markata import Markata


class SilentUndefined(Undefined):
    def _fail_with_undefined_error(self, *args, **kwargs):
        return ""


def has_rich_handler() -> bool:
    """
    Returns a boolean whether or not there is a RichHandler attached to the
    root logger.
    """

    logger = logging.getLogger()
    return bool([h for h in logger.handlers if isinstance(h, RichHandler)])


def has_file_handler(log_file: Path) -> bool:
    logger = logging.getLogger()
    existing_logger_files = [
        handler.baseFilename
        for handler in logger.handlers
        if isinstance(handler, logging.FileHandler)
    ]
    return str(log_file.absolute()) in existing_logger_files


def setup_log(markata: "Markata", level: int = logging.INFO) -> Path:
    path = setup_html_log(markata, level)
    setup_text_log(markata, level)
    return path


def setup_text_log(markata: "Markata", level: int = logging.INFO) -> Path:
    """
    sets up a plain text log in markata's configured `log_dir`, or
    `output_dir/_logs` if `log_dir` is not configured.  The log file will be
    named after the `<levelname>.log`
    """
    log_file = Path(
        str(
            markata.config.get(
                "log_dir",
                Path(str(markata.config.get("output_dir", "markout")))
                / "_logs"
                / (logging.getLevelName(level).lower() + ".log"),
            )
        )
    )
    if has_file_handler(log_file):
        return log_file

    if not log_file.parent.exists():
        log_file.parent.mkdir(parents=True)
    fh = logging.FileHandler(log_file)
    fh.setLevel(level)
    fh_formatter = logging.Formatter(
        "%(asctime)s %(name)-12s %(levelname)-8s %(message)s",
    )
    fh.setFormatter(fh_formatter)
    logging.getLogger("").addHandler(fh)

    return log_file


def setup_html_log(markata: "Markata", level: int = logging.INFO) -> Path:
    """
    sets up an html log in markata's configured `log_dir`, or
    `output_dir/_logs` if `log_dir` is not configured.  The log file will be
    named after the `<levelname>/index.html`.  The goal of this is to give
    """

    log_file = Path(
        str(
            markata.config.get(
                "log_dir",
                Path(str(markata.config.get("output_dir", "markout")))
                / "_logs"
                / logging.getLevelName(level).lower()
                / "index.html",
            )
        )
    )

    if has_file_handler(log_file):
        return log_file

    if not log_file.parent.exists():
        log_file.parent.mkdir(parents=True)
    if not log_file.exists():
        template_file = Path(
            str(
                markata.config.get(
                    "log_template", Path(__file__).parent / "default_log_template.html"
                )
            )
        )
        template = Template(template_file.read_text(), undefined=SilentUndefined)
        log_header = template.render(
            title=str(markata.config.get("title", "markata build")) + " logs",
            config=markata.config,
        )
        log_file.write_text(log_header)
    with open(log_file, "a") as f:
        command = Path(sys.argv[0]).name + " " + " ".join(sys.argv[1:])
        f.write(
            f"""
            <div style="width: 100%; height: 20px; margin-top: 5rem; border-bottom: 1px solid goldenrod; text-align: center">
    <span style="padding: 0 10px;">
    {datetime.datetime.now()} running "{command}"
    </span>
    </div>
    """
        )
    fh = logging.FileHandler(log_file)
    fh.setLevel(level)
    fh_formatter = logging.Formatter(
        "<li><p><span class='time'>%(asctime)s</span> <span class='name %(name)s'>%(name)-12s</span> <span class='levelname %(levelname)s'>%(levelname)-8s</span></p><p class='message'>%(message)s</p></li>"
    )
    fh.setFormatter(fh_formatter)
    logging.getLogger("").addHandler(fh)

    return log_file


@hook_impl(tryfirst=True)
def configure(markata: "Markata") -> None:
    setup_log(markata, logging.DEBUG)
    setup_log(markata, logging.INFO)
    setup_log(markata, logging.WARNING)

    if not has_rich_handler():
        console = RichHandler(rich_tracebacks=True)
        console.setLevel(logging.INFO)
        formatter = logging.Formatter("%(message)s")
        console.setFormatter(formatter)
        logging.getLogger("").addHandler(console)



import os
import textwrap
from typing import Any

import pytest

from markata import Markata


@pytest.fixture(scope="session")
def make_project(tmp_path_factory: Any) -> Any:
    project = tmp_path_factory.mktemp("project")
    module = project / "my_module.py"
    module.write_text(
        textwrap.dedent(
            """
            '''
            Module level docstring
            '''

            def my_func():
                '''
                docstring for my_func
                '''
            class MyClass:
                '''
                docstring for MyClass
                '''

                def my_method(self):
                    '''
                    docstring for my_method
                    '''

            """
        )
    )
    markta_toml = project / "markata.toml"
    markta_toml.write_text(
        textwrap.dedent(
            """
            [markata]
            hooks = [
                "markata.plugins.docs",
                "default",
                ]
            """
        )
    )

    return project


def test_loaded(make_project: Any) -> None:
    os.chdir(make_project)
    m = Markata()
    assert len(m.py_files) == 1


@pytest.fixture(scope="session")
def test_run(make_project: Any) -> Any:
    os.chdir(make_project)
    m = Markata()
    m.run()
    return make_project


def test_markout_exists(test_run: Any) -> Any:
    markout = test_run / "markout"
    assert markout.exists()


def test_index_exists(test_run: Any) -> Any:
    markout = test_run / "markout"
    index = markout / "my-module" / "index.html"
    assert index.exists()


def test_rss_exists(test_run: Any) -> Any:
    markout = test_run / "markout"
    rss = markout / "rss.xml"
    assert rss.exists()



"""
Slugify migration for projects moving from markata<0.5.0 into markata>=0.5.0

to run this script install markata>=0.5.0 and run the following.

``` bash
python -m markata.scripts.migrate_to_slugify
```

Then make sure that you do not explicity turn off slugify and your site is
going to be on to better urls.

``` toml
[markata]
slugify=false
```
"""
from pathlib import Path

from slugify import slugify

from markata import Markata


def routed_slugify(text):
    return "/".join([slugify(s) for s in text.split("/")])


if __name__ == "__main__":
    m = Markata()
    m.config["slugify"] = False

    original_urls = m.map("slug")
    redirects = [
        "/" + o.ljust(60) + "/" + routed_slugify(o)
        for o in original_urls
        if routed_slugify(o) != o
    ]
    assets_dir: str = str(m.config.get("assets_dir", "static"))
    redirects_file = Path(
        str(m.config.get("redirects", Path(assets_dir) / "_redirects"))
    )
    redirects_file.touch()
    with open(redirects_file, "a") as f:
        f.write("\n")
        f.write("# marakta migrate to slugify redirects\n")
        f.write("\n".join(redirects))
        f.write("\n")



import os
import textwrap
from typing import Any

import pytest

from markata import Markata


@pytest.fixture(scope="session")
def make_index(tmp_path_factory: Any) -> Any:
    fn = tmp_path_factory.mktemp("pages") / "index.md"
    fn.write_text(
        textwrap.dedent(
            """
            ---
            templateKey: blog-post
            tags: ['python',]
            title:  My Awesome Post
            date: 2022-01-21T16:40:34
            status: draft

            ---

            This is my awesome post.
            """
        )
    )
    return tmp_path_factory


def test_loaded(make_index: Any) -> None:
    os.chdir(make_index.getbasetemp())
    m = Markata()
    assert len(m.articles) == 1


@pytest.fixture(scope="session")
def test_run(make_index: Any) -> Any:
    os.chdir(make_index.getbasetemp())
    m = Markata()
    m.run()
    return make_index


def test_markout_exists(test_run: Any) -> Any:
    markout = test_run.getbasetemp() / "markout"
    assert markout.exists()


def test_index_exists(test_run: Any) -> Any:
    markout = test_run.getbasetemp() / "markout"
    index = markout / "index.html"
    assert index.exists()


def test_rss_exists(test_run: Any) -> Any:
    markout = test_run.getbasetemp() / "markout"
    rss = markout / "rss.xml"
    assert rss.exists()
